@@ -193,7 +193,7 @@ empathy_contact_information_dialog_show (EmpathyContact *contact,
 		EMPATHY_CONTACT_WIDGET_SHOW_LOCATION |
 		EMPATHY_CONTACT_WIDGET_EDIT_NONE);
 	gtk_container_set_border_width (GTK_CONTAINER (contact_widget), 8);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
 			    contact_widget,
 			    TRUE, TRUE, 0);
 	gtk_widget_show (contact_widget);
@@ -252,7 +252,7 @@ empathy_contact_edit_dialog_show (EmpathyContact *contact,
 		EMPATHY_CONTACT_WIDGET_EDIT_ALIAS |
 		EMPATHY_CONTACT_WIDGET_EDIT_GROUPS);
 	gtk_container_set_border_width (GTK_CONTAINER (contact_widget), 8);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
 			    contact_widget,
 			    TRUE, TRUE, 0);
 	gtk_widget_show (contact_widget);
@@ -304,7 +304,7 @@ empathy_contact_personal_dialog_show (GtkWindow *parent)
 		EMPATHY_CONTACT_WIDGET_EDIT_ALIAS |
 		EMPATHY_CONTACT_WIDGET_EDIT_AVATAR);
 	gtk_container_set_border_width (GTK_CONTAINER (contact_widget), 8);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (personal_dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (personal_dialog))),
 			    contact_widget,
 			    TRUE, TRUE, 0);
 	empathy_contact_widget_set_account_filter (contact_widget,
@@ -328,26 +328,20 @@ empathy_contact_personal_dialog_show (GtkWindow *parent)
  */
 
 static gboolean
-can_add_contact_to_account (McAccount *account,
+can_add_contact_to_account (EmpathyAccount *account,
 			    gpointer   user_data)
 {
-	EmpathyAccountManager *account_manager;
 	EmpathyContactManager *contact_manager;
 	TpConnection          *connection;
 	gboolean               result;
 
-	account_manager = empathy_account_manager_dup_singleton ();
-	connection = empathy_account_manager_get_connection (account_manager,
-							     account);
-	if (!connection) {
-		g_object_unref (account_manager);
+	connection = empathy_account_get_connection (account);
+	if (connection == NULL)
 		return FALSE;
-	}
 
 	contact_manager = empathy_contact_manager_dup_singleton ();
-	result = empathy_contact_manager_can_add (contact_manager, connection);
+	result = empathy_contact_manager_get_flags_for_connection (contact_manager, connection) & EMPATHY_CONTACT_LIST_CAN_ADD;
 	g_object_unref (contact_manager);
-	g_object_unref (account_manager);
 
 	return result;
 }
@@ -375,6 +369,13 @@ new_contact_response_cb (GtkDialog *dialog,
 
 void
 empathy_new_contact_dialog_show (GtkWindow *parent)
+{
+	empathy_new_contact_dialog_show_with_contact (parent, NULL);
+}
+
+void
+empathy_new_contact_dialog_show_with_contact (GtkWindow *parent,
+                                              EmpathyContact *contact)
 {
 	GtkWidget *dialog;
 	GtkWidget *button;
@@ -398,7 +399,7 @@ empathy_new_contact_dialog_show (GtkWindow *parent)
 				      button,
 				      GTK_RESPONSE_CANCEL);
 	gtk_widget_show (button);
-	
+
 	/* Add button */
 	button = gtk_button_new_with_label (GTK_STOCK_ADD);
 	gtk_button_set_use_stock (GTK_BUTTON (button), TRUE);
@@ -408,12 +409,12 @@ empathy_new_contact_dialog_show (GtkWindow *parent)
 	gtk_widget_show (button);
 
 	/* Contact info widget */
-	contact_widget = empathy_contact_widget_new (NULL,
+	contact_widget = empathy_contact_widget_new (contact,
 						     EMPATHY_CONTACT_WIDGET_EDIT_ALIAS |
 						     EMPATHY_CONTACT_WIDGET_EDIT_ACCOUNT |
 						     EMPATHY_CONTACT_WIDGET_EDIT_ID |
 						     EMPATHY_CONTACT_WIDGET_EDIT_GROUPS);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
 			    contact_widget,
 			    TRUE, TRUE, 0);
 	empathy_contact_widget_set_account_filter (contact_widget,
