@@ -33,6 +33,9 @@
 #include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-enum-types.h>
+#include <libempathy/empathy-contact-manager.h>
+
 #include "empathy-contact-list-store.h"
 #include "empathy-ui-utils.h"
 #include "empathy-gtk-enum-types.h"
@@ -601,7 +604,7 @@ empathy_contact_list_store_set_sort_criterium (EmpathyContactListStore     *stor
 						      EMPATHY_CONTACT_LIST_STORE_COL_STATUS,
 						      GTK_SORT_ASCENDING);
 		break;
-		
+
 	case EMPATHY_CONTACT_LIST_STORE_SORT_NAME:
 		gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
 						      EMPATHY_CONTACT_LIST_STORE_COL_NAME,
@@ -725,20 +728,23 @@ static void
 contact_list_store_setup (EmpathyContactListStore *store)
 {
 	EmpathyContactListStorePriv *priv;
-	GType                       types[] = {G_TYPE_STRING,        /* Status icon-name */
-					       GDK_TYPE_PIXBUF,      /* Avatar pixbuf */
-					       G_TYPE_BOOLEAN,       /* Avatar pixbuf visible */
-					       G_TYPE_STRING,        /* Name */
-					       G_TYPE_STRING,        /* Status string */
-					       G_TYPE_BOOLEAN,       /* Show status */
-					       EMPATHY_TYPE_CONTACT, /* Contact type */
-					       G_TYPE_BOOLEAN,       /* Is group */
-					       G_TYPE_BOOLEAN,       /* Is active */
-					       G_TYPE_BOOLEAN,       /* Is online */
-					       G_TYPE_BOOLEAN,       /* Is separator */
-					       G_TYPE_BOOLEAN,       /* Can make audio calls */
-					       G_TYPE_BOOLEAN};      /* Can make video calls */
-	
+	GType types[] = {
+		G_TYPE_STRING,        /* Status icon-name */
+		GDK_TYPE_PIXBUF,      /* Avatar pixbuf */
+		G_TYPE_BOOLEAN,       /* Avatar pixbuf visible */
+		G_TYPE_STRING,        /* Name */
+		G_TYPE_STRING,        /* Status string */
+		G_TYPE_BOOLEAN,       /* Show status */
+		EMPATHY_TYPE_CONTACT, /* Contact type */
+		G_TYPE_BOOLEAN,       /* Is group */
+		G_TYPE_BOOLEAN,       /* Is active */
+		G_TYPE_BOOLEAN,       /* Is online */
+		G_TYPE_BOOLEAN,       /* Is separator */
+		G_TYPE_BOOLEAN,       /* Can make audio calls */
+		G_TYPE_BOOLEAN,       /* Can make video calls */
+		EMPATHY_TYPE_CONTACT_LIST_FLAGS, /* Flags */
+	};
+
 	priv = GET_PRIV (store);
 
 	gtk_tree_store_set_column_types (GTK_TREE_STORE (store),
@@ -851,9 +857,11 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 	EmpathyContactListStorePriv *priv;
 	GtkTreeIter                 iter;
 	GList                      *groups = NULL, *l;
+	TpConnection               *connection;
+	EmpathyContactListFlags     flags;
 
 	priv = GET_PRIV (store);
-	
+
 	if (EMP_STR_EMPTY (empathy_contact_get_name (contact)) ||
 	    (!priv->show_offline && !empathy_contact_is_online (contact))) {
 		return;
@@ -863,6 +871,14 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 		groups = empathy_contact_list_get_groups (priv->list, contact);
 	}
 
+	connection = empathy_contact_get_connection (contact);
+	if (EMPATHY_IS_CONTACT_MANAGER (priv->list)) {
+		flags = empathy_contact_manager_get_flags_for_connection (
+			EMPATHY_CONTACT_MANAGER (priv->list), connection);
+	} else {
+		flags = 0;
+	}
+
 	/* If no groups just add it at the top level. */
 	if (!groups) {
 		gtk_tree_store_append (GTK_TREE_STORE (store), &iter, NULL);
@@ -877,6 +893,7 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
 				      empathy_contact_get_capabilities (contact) &
 				        EMPATHY_CAPABILITIES_VIDEO,
+				    EMPATHY_CONTACT_LIST_STORE_COL_FLAGS, flags,
 				    -1);
 	}
 
@@ -899,6 +916,7 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 				    EMPATHY_CONTACT_LIST_STORE_COL_CAN_VIDEO_CALL,
 				      empathy_contact_get_capabilities (contact) &
 				        EMPATHY_CAPABILITIES_VIDEO,
+				    EMPATHY_CONTACT_LIST_STORE_COL_FLAGS, flags,
 				    -1);
 		g_free (l->data);
 	}
@@ -922,7 +940,7 @@ contact_list_store_remove_contact (EmpathyContactListStore *store,
 	if (!iters) {
 		return;
 	}
-	
+
 	/* Clean up model */
 	model = GTK_TREE_MODEL (store);
 
@@ -1156,7 +1174,7 @@ contact_list_store_contact_active_new (EmpathyContactListStore *store,
 	DEBUG ("Contact:'%s' now active, and %s be removed",
 		empathy_contact_get_name (contact),
 		remove ? "WILL" : "WILL NOT");
-	
+
 	data = g_slice_new0 (ShowActiveData);
 
 	data->store = g_object_ref (store);
