@@ -25,46 +25,73 @@
 #include <glib/gi18n.h>
 
 #include <webkit/webkitnetworkrequest.h>
+#include <telepathy-glib/dbus.h>
+#include <telepathy-glib/util.h>
+
 
 #include <libempathy/empathy-time.h>
 #include <libempathy/empathy-utils.h>
+#include <libmissioncontrol/mc-profile.h>
 
 #include "empathy-theme-adium.h"
 #include "empathy-smiley-manager.h"
 #include "empathy-conf.h"
 #include "empathy-ui-utils.h"
+#include "empathy-plist.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyThemeAdium)
 
+/* "Join" consecutive messages with timestamps within five minutes */
+#define MESSAGE_JOIN_PERIOD 5*60
+
 typedef struct {
+	EmpathyAdiumData     *data;
 	EmpathySmileyManager *smiley_manager;
 	EmpathyContact       *last_contact;
+	time_t                last_timestamp;
+	gboolean              last_is_backlog;
 	gboolean              page_loaded;
 	GList                *message_queue;
-	gchar                *path;
-	gchar                *default_avatar_filename;
-	gchar                *template_html;
-	gchar                *basedir;
-	gchar                *in_content_html;
-	gsize                 in_content_len;
-	gchar                *in_nextcontent_html;
-	gsize                 in_nextcontent_len;
-	gchar                *out_content_html;
-	gsize                 out_content_len;
-	gchar                *out_nextcontent_html;
-	gsize                 out_nextcontent_len;
-	gchar                *status_html;
-	gsize                 status_len;
+	gchar                *hovered_uri;
 } EmpathyThemeAdiumPriv;
 
+struct _EmpathyAdiumData {
+	guint  ref_count;
+	gchar *path;
+	gchar *basedir;
+	gchar *default_avatar_filename;
+	gchar *default_incoming_avatar_filename;
+	gchar *default_outgoing_avatar_filename;
+	gchar *template_html;
+	gchar *in_content_html;
+	gsize  in_content_len;
+	gchar *in_context_html;
+	gsize  in_context_len;
+	gchar *in_nextcontent_html;
+	gsize  in_nextcontent_len;
+	gchar *in_nextcontext_html;
+	gsize  in_nextcontext_len;
+	gchar *out_content_html;
+	gsize  out_content_len;
+	gchar *out_context_html;
+	gsize  out_context_len;
+	gchar *out_nextcontent_html;
+	gsize  out_nextcontent_len;
+	gchar *out_nextcontext_html;
+	gsize  out_nextcontext_len;
+	gchar *status_html;
+	gsize  status_len;
+	GHashTable *info;
+};
+
 static void theme_adium_iface_init (EmpathyChatViewIface *iface);
 
 enum {
 	PROP_0,
-	PROP_PATH,
+	PROP_ADIUM_DATA,
 };
 
 G_DEFINE_TYPE_WITH_CODE (EmpathyThemeAdium, empathy_theme_adium,
@@ -72,100 +99,6 @@ G_DEFINE_TYPE_WITH_CODE (EmpathyThemeAdium, empathy_theme_adium,
 			 G_IMPLEMENT_INTERFACE (EMPATHY_TYPE_CHAT_VIEW,
 						theme_adium_iface_init));
 
-static void
-theme_adium_load (EmpathyThemeAdium *theme)
-{
-	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
-	gchar                 *file;
-	gchar                 *template_html = NULL;
-	gsize                  template_len;
-	GString               *string;
-	gchar                **strv = NULL;
-	gchar                 *css_path;
-	guint                  len = 0;
-	guint                  i = 0;
-	gchar                 *basedir_uri;
-
-	priv->basedir = g_strconcat (priv->path, G_DIR_SEPARATOR_S "Contents" G_DIR_SEPARATOR_S "Resources" G_DIR_SEPARATOR_S, NULL);
-	basedir_uri = g_strconcat ("file://", priv->basedir, NULL);
-
-	/* Load html files */
-	file = g_build_filename (priv->basedir, "Incoming", "Content.html", NULL);
-	g_file_get_contents (file, &priv->in_content_html, &priv->in_content_len, NULL);
-	g_free (file);
-
-	file = g_build_filename (priv->basedir, "Incoming", "NextContent.html", NULL);
-	g_file_get_contents (file, &priv->in_nextcontent_html, &priv->in_nextcontent_len, NULL);
-	g_free (file);
-
-	file = g_build_filename (priv->basedir, "Outgoing", "Content.html", NULL);
-	g_file_get_contents (file, &priv->out_content_html, &priv->out_content_len, NULL);
-	g_free (file);
-
-	file = g_build_filename (priv->basedir, "Outgoing", "NextContent.html", NULL);
-	g_file_get_contents (file, &priv->out_nextcontent_html, &priv->out_nextcontent_len, NULL);
-	g_free (file);
-
-	file = g_build_filename (priv->basedir, "Status.html", NULL);
-	g_file_get_contents (file, &priv->status_html, &priv->status_len, NULL);
-	g_free (file);
-
-	css_path = g_build_filename (priv->basedir, "main.css", NULL);
-
-	/* There is 2 formats for Template.html: The old one has 4 parameters,
-	 * the new one has 5 parameters. */
-	file = g_build_filename (priv->basedir, "Template.html", NULL);
-	if (g_file_get_contents (file, &template_html, &template_len, NULL)) {
-		strv = g_strsplit (template_html, "%@", -1);
-		len = g_strv_length (strv);
-	}
-	g_free (file);
-
-	if (len != 5 && len != 6) {
-		/* Either the theme has no template or it don't have the good
-		 * number of parameters. Fallback to use our own template. */
-		g_free (template_html);
-		g_strfreev (strv);
-
-		file = empathy_file_lookup ("Template.html", "data");
-		g_file_get_contents (file, &template_html, &template_len, NULL);
-		g_free (file);
-		strv = g_strsplit (template_html, "%@", -1);
-		len = g_strv_length (strv);
-	}
-
-	/* Replace %@ with the needed information in the template html. */
-	string = g_string_sized_new (template_len);
-	g_string_append (string, strv[i++]);
-	g_string_append (string, priv->basedir);
-	g_string_append (string, strv[i++]);
-	if (len == 6) {
-		/* We include main.css by default */
-		g_string_append_printf (string, "@import url(\"%s\");", css_path);
-		g_string_append (string, strv[i++]);
-		/* FIXME: We should set the variant css here */
-		g_string_append (string, "");
-	} else {
-		/* FIXME: We should set main.css OR the variant css */
-		g_string_append (string, css_path);
-	}
-	g_string_append (string, strv[i++]);
-	g_string_append (string, ""); /* We don't want header */
-	g_string_append (string, strv[i++]);
-	g_string_append (string, ""); /* FIXME: We don't support footer yet */
-	g_string_append (string, strv[i++]);
-	priv->template_html = g_string_free (string, FALSE);
-
-	/* Load the template */
-	webkit_web_view_load_html_string (WEBKIT_WEB_VIEW (theme),
-					  priv->template_html, basedir_uri);
-
-	g_free (basedir_uri);
-	g_free (template_html);
-	g_free (css_path);
-	g_strfreev (strv);
-}
-
 static WebKitNavigationResponse
 theme_adium_navigation_requested_cb (WebKitWebView        *view,
 				     WebKitWebFrame       *frame,
@@ -180,22 +113,82 @@ theme_adium_navigation_requested_cb (WebKitWebView        *view,
 	return WEBKIT_NAVIGATION_RESPONSE_IGNORE;
 }
 
+static void
+theme_adium_hovering_over_link_cb (EmpathyThemeAdium *theme,
+				   gchar             *title,
+				   gchar             *uri,
+				   gpointer           user_data)
+{
+	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
+
+	if (tp_strdiff (uri, priv->hovered_uri)) {
+		g_free (priv->hovered_uri);
+		priv->hovered_uri = g_strdup (uri);
+	}
+}
+
+static void
+theme_adium_copy_address_cb (GtkMenuItem *menuitem,
+			     gpointer     user_data)
+{
+	EmpathyThemeAdium     *theme = EMPATHY_THEME_ADIUM (user_data);
+	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
+	GtkClipboard          *clipboard;
+
+	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text (clipboard, priv->hovered_uri, -1);
+
+	clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
+	gtk_clipboard_set_text (clipboard, priv->hovered_uri, -1);
+}
+
+static void
+theme_adium_open_address_cb (GtkMenuItem *menuitem,
+			     gpointer     user_data)
+{
+	EmpathyThemeAdium     *theme = EMPATHY_THEME_ADIUM (user_data);
+	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
+
+	empathy_url_show (GTK_WIDGET (menuitem), priv->hovered_uri);
+}
+
 static void
 theme_adium_populate_popup_cb (WebKitWebView *view,
 			       GtkMenu       *menu,
 			       gpointer       user_data)
 {
 	GtkWidget *item;
+	GList     *items;
+	GtkWidget *icon;
+	gchar     *stock_id;
+	gboolean   is_link = FALSE;
+
+	/* FIXME: WebKitGTK+'s context menu API clearly needs an
+	 * overhaul.  There is currently no way to know what is being
+	 * clicked, to decide what features to provide. You either
+	 * take what it gives you as a menu, or use hacks to figure
+	 * out what to display. */
+	items = gtk_container_get_children (GTK_CONTAINER (menu));
+	item = GTK_WIDGET (g_list_nth_data (items, 0));
+	g_list_free (items);
+
+	if (GTK_IS_IMAGE_MENU_ITEM (item)) {
+		icon = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (item));
+		gtk_image_get_stock (GTK_IMAGE (icon), &stock_id, NULL);
+
+		if (!strcmp (stock_id, GTK_STOCK_OPEN))
+			is_link = TRUE;
+	}
 
 	/* Remove default menu items */
 	gtk_container_foreach (GTK_CONTAINER (menu),
 		(GtkCallback) gtk_widget_destroy, NULL);
-	
+
 	/* Select all item */
 	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_SELECT_ALL, NULL);
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
-		
+
 	g_signal_connect_swapped (item, "activate",
 				  G_CALLBACK (webkit_web_view_select_all),
 				  view);
@@ -205,7 +198,7 @@ theme_adium_populate_popup_cb (WebKitWebView *view,
 		item = gtk_image_menu_item_new_from_stock (GTK_STOCK_COPY, NULL);
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
-		
+
 		g_signal_connect_swapped (item, "activate",
 					  G_CALLBACK (webkit_web_view_copy_clipboard),
 					  view);
@@ -215,19 +208,40 @@ theme_adium_populate_popup_cb (WebKitWebView *view,
 	item = gtk_separator_menu_item_new ();
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
-		
+
 	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_CLEAR, NULL);
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
-		
+
 	g_signal_connect_swapped (item, "activate",
 				  G_CALLBACK (empathy_chat_view_clear),
 				  view);
 
-	/* FIXME: Add open_link and copy_link when those bugs are fixed:
-	 * https://bugs.webkit.org/show_bug.cgi?id=16092
-	 * https://bugs.webkit.org/show_bug.cgi?id=16562
-	 */
+	/* We will only add the following menu items if we are
+	 * right-clicking a link */
+	if (!is_link)
+		return;
+
+	/* Separator */
+	item = gtk_separator_menu_item_new ();
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show (item);
+
+	/* Copy Link Address menu item */
+	item = gtk_menu_item_new_with_mnemonic (_("_Copy Link Address"));
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (theme_adium_copy_address_cb),
+			  view);
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show (item);
+
+	/* Open Link menu item */
+	item = gtk_menu_item_new_with_mnemonic (_("_Open Link"));
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (theme_adium_open_address_cb),
+			  view);
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	gtk_widget_show (item);
 }
 
 static gchar *
@@ -343,7 +357,7 @@ escape_and_append_len (GString *string, const gchar *str, gint len)
 		switch (*str) {
 		case '\\':
 			/* \ becomes \\ */
-			g_string_append (string, "\\\\");	
+			g_string_append (string, "\\\\");
 			break;
 		case '\"':
 			/* " becomes \" */
@@ -382,6 +396,9 @@ theme_adium_append_html (EmpathyThemeAdium *theme,
 		         const gchar       *message,
 		         const gchar       *avatar_filename,
 		         const gchar       *name,
+		         const gchar       *contact_id,
+		         const gchar       *service_name,
+		         const gchar       *message_classes,
 		         time_t             timestamp)
 {
 	GString     *string;
@@ -397,10 +414,28 @@ theme_adium_append_html (EmpathyThemeAdium *theme,
 
 		if (theme_adium_match (&cur, "%message%")) {
 			replace = message;
+		} else if (theme_adium_match (&cur, "%messageClasses%")) {
+			replace = message_classes;
 		} else if (theme_adium_match (&cur, "%userIconPath%")) {
 			replace = avatar_filename;
 		} else if (theme_adium_match (&cur, "%sender%")) {
 			replace = name;
+		} else if (theme_adium_match (&cur, "%senderScreenName%")) {
+			replace = contact_id;
+		} else if (theme_adium_match (&cur, "%senderDisplayName%")) {
+			/* %senderDisplayName% -
+			 * "The serverside (remotely set) name of the sender,
+			 *  such as an MSN display name."
+			 *
+			 * We don't have access to that yet so we use local
+			 * alias instead.*/
+			replace = name;
+		} else if (theme_adium_match (&cur, "%service%")) {
+			replace = service_name;
+		} else if (theme_adium_match (&cur, "%shortTime%")) {
+			dup_replace = empathy_time_to_string_local (timestamp,
+				EMPATHY_TIME_FORMAT_DISPLAY_SHORT);
+			replace = dup_replace;
 		} else if (theme_adium_match (&cur, "%time")) {
 			gchar *format = NULL;
 			gchar *end;
@@ -448,15 +483,21 @@ theme_adium_append_message (EmpathyChatView *view,
 	EmpathyThemeAdium     *theme = EMPATHY_THEME_ADIUM (view);
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
 	EmpathyContact        *sender;
+	EmpathyAccount        *account;
+	McProfile             *account_profile;
 	gchar                 *dup_body = NULL;
 	const gchar           *body;
 	const gchar           *name;
+	const gchar           *contact_id;
 	EmpathyAvatar         *avatar;
 	const gchar           *avatar_filename = NULL;
 	time_t                 timestamp;
 	gchar                 *html = NULL;
 	gsize                  len = 0;
 	const gchar           *func;
+	const gchar           *service_name;
+	GString               *message_classes = NULL;
+	gboolean              is_backlog;
 
 	if (!priv->page_loaded) {
 		priv->message_queue = g_list_prepend (priv->message_queue,
@@ -466,6 +507,9 @@ theme_adium_append_message (EmpathyChatView *view,
 
 	/* Get information */
 	sender = empathy_message_get_sender (msg);
+	account = empathy_contact_get_account (sender);
+	account_profile = empathy_account_get_profile (account);
+	service_name = mc_profile_get_display_name (account_profile);
 	timestamp = empathy_message_get_timestamp (msg);
 	body = empathy_message_get_body (msg);
 	dup_body = theme_adium_parse_body (theme, body);
@@ -473,6 +517,7 @@ theme_adium_append_message (EmpathyChatView *view,
 		body = dup_body;
 	}
 	name = empathy_contact_get_name (sender);
+	contact_id = empathy_contact_get_id (sender);
 
 	/* If this is a /me, append an event */
 	if (empathy_message_get_tptype (msg) == TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION) {
@@ -491,48 +536,129 @@ theme_adium_append_message (EmpathyChatView *view,
 		avatar_filename = avatar->filename;
 	}
 	if (!avatar_filename) {
-		if (!priv->default_avatar_filename) {
-			priv->default_avatar_filename =
-				empathy_filename_from_icon_name ("stock_person",
-								 GTK_ICON_SIZE_DIALOG);
+		if (empathy_contact_is_user (sender)) {
+			avatar_filename = priv->data->default_outgoing_avatar_filename;
+		} else {
+			avatar_filename = priv->data->default_incoming_avatar_filename;
+		}
+		if (!avatar_filename) {
+			if (!priv->data->default_avatar_filename) {
+				priv->data->default_avatar_filename =
+					empathy_filename_from_icon_name ("stock_person",
+									 GTK_ICON_SIZE_DIALOG);
+			}
+			avatar_filename = priv->data->default_avatar_filename;
 		}
-		avatar_filename = priv->default_avatar_filename;
 	}
 
+	is_backlog = empathy_message_is_backlog (msg);
+
 	/* Get the right html/func to add the message */
-	if (priv->last_contact == sender) {
+	func = "appendMessage";
+
+	message_classes = g_string_new ("message");
+
+	/* eventually append the "history" class */
+	if (is_backlog) {
+		g_string_append (message_classes, " history");
+	}
+
+	/* check the sender of the message and append the appropriate class */
+	if (empathy_contact_is_user (sender)) {
+		g_string_append (message_classes, " outgoing");
+	}
+	else {
+		g_string_append (message_classes, " incoming");
+	}
+
+	/*
+	 * To mimick Adium's behavior, we only want to join messages
+	 * sent by the same contact within a 5 minute time frame.
+	 */
+	if (empathy_contact_equal (priv->last_contact, sender) &&
+	    (timestamp - priv->last_timestamp < MESSAGE_JOIN_PERIOD) &&
+	    (is_backlog == priv->last_is_backlog)) {
+		/* the messages can be appended */
 		func = "appendNextMessage";
+		g_string_append (message_classes, " consecutive");
+
+		/* check who is the sender of the message to use the correct html file */
 		if (empathy_contact_is_user (sender)) {
-			html = priv->out_nextcontent_html;
-			len = priv->out_nextcontent_len;
+			/* check if this is a backlog message and use NextContext.html */
+			if (is_backlog) {
+				html = priv->data->out_nextcontext_html;
+				len = priv->data->out_nextcontext_len;
+			}
+
+			/*
+			 * html is null if this is not a backlog message or
+			 * if we have to fallback (NextContext.html missing).
+			 * use NextContent.html
+			 */
+			if (html == NULL) {
+				html = priv->data->out_nextcontent_html;
+				len = priv->data->out_nextcontent_len;
+			}
 		}
-		if (!html) {
-			html = priv->in_nextcontent_html;
-			len = priv->in_nextcontent_len;
+		else {
+			if (is_backlog) {
+				html = priv->data->in_nextcontext_html;
+				len = priv->data->in_nextcontext_len;
+			}
+
+			if (html == NULL) {
+				html = priv->data->in_nextcontent_html;
+				len = priv->data->in_nextcontent_len;
+			}
 		}
 	}
-	if (!html) {
-		func = "appendMessage";
+
+	/*
+	 * we have html == NULL here if:
+	 * 1. the message didn't have to be appended because
+	 *    the sender was different or the timestamp was too far
+	 * 2. NextContent.html file does not exist, so we must
+	 *    not forget to fallback to the correct Content.html
+	 */
+	if (html == NULL) {
 		if (empathy_contact_is_user (sender)) {
-			html = priv->out_content_html;
-			len = priv->out_content_len;
+			if (is_backlog) {
+				html = priv->data->out_context_html;
+				len = priv->data->out_context_len;
+			}
+
+			if (html == NULL) {
+				html = priv->data->out_content_html;
+				len = priv->data->out_content_len;
+			}
 		}
-		if (!html) {
-			html = priv->in_content_html;
-			len = priv->in_content_len;
+		else {
+			if (is_backlog) {
+				html = priv->data->in_context_html;
+				len = priv->data->in_context_len;
+			}
+
+			if (html == NULL) {
+				html = priv->data->in_content_html;
+				len = priv->data->in_content_len;
+			}
 		}
 	}
 
 	theme_adium_append_html (theme, func, html, len, body, avatar_filename,
-				 name, timestamp);
+				 name, contact_id, service_name, message_classes->str,
+				 timestamp);
 
 	/* Keep the sender of the last displayed message */
 	if (priv->last_contact) {
 		g_object_unref (priv->last_contact);
 	}
 	priv->last_contact = g_object_ref (sender);
+	priv->last_timestamp = timestamp;
+	priv->last_is_backlog = is_backlog;
 
 	g_free (dup_body);
+	g_string_free (message_classes, TRUE);
 }
 
 static void
@@ -542,10 +668,11 @@ theme_adium_append_event (EmpathyChatView *view,
 	EmpathyThemeAdium     *theme = EMPATHY_THEME_ADIUM (view);
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
 
-	if (priv->status_html) {
+	if (priv->data->status_html) {
 		theme_adium_append_html (theme, "appendMessage",
-					 priv->status_html, priv->status_len,
-					 str, NULL, NULL,
+					 priv->data->status_html,
+					 priv->data->status_len,
+					 str, NULL, NULL, NULL, NULL, "event",
 					 empathy_time_get_current ());
 	}
 
@@ -583,10 +710,18 @@ theme_adium_clear (EmpathyChatView *view)
 	gchar *basedir_uri;
 
 	priv->page_loaded = FALSE;
-	basedir_uri = g_strconcat ("file://", priv->basedir, NULL);
+	basedir_uri = g_strconcat ("file://", priv->data->basedir, NULL);
 	webkit_web_view_load_html_string (WEBKIT_WEB_VIEW (view),
-					  priv->template_html, basedir_uri);
+					  priv->data->template_html,
+					  basedir_uri);
 	g_free (basedir_uri);
+
+	/* Clear last contact to avoid trying to add a 'joined'
+	 * message when we don't have an insertion point. */
+	if (priv->last_contact) {
+		g_object_unref (priv->last_contact);
+		priv->last_contact = NULL;
+	}
 }
 
 static gboolean
@@ -683,14 +818,8 @@ theme_adium_finalize (GObject *object)
 {
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (object);
 
-	g_free (priv->basedir);
-	g_free (priv->template_html);
-	g_free (priv->in_content_html);
-	g_free (priv->in_nextcontent_html);
-	g_free (priv->out_content_html);
-	g_free (priv->out_nextcontent_html);
-	g_free (priv->default_avatar_filename);
-	g_free (priv->path);
+	empathy_adium_data_unref (priv->data);
+	g_free (priv->hovered_uri);
 
 	G_OBJECT_CLASS (empathy_theme_adium_parent_class)->finalize (object);
 }
@@ -716,7 +845,32 @@ theme_adium_dispose (GObject *object)
 static void
 theme_adium_constructed (GObject *object)
 {
-	theme_adium_load (EMPATHY_THEME_ADIUM (object));
+	EmpathyThemeAdiumPriv *priv = GET_PRIV (object);
+	gchar                 *basedir_uri;
+	const gchar           *font_family = NULL;
+	gint                   font_size = 0;
+	WebKitWebSettings     *webkit_settings;
+
+	/* Set default settings */
+	font_family = tp_asv_get_string (priv->data->info, "DefaultFontFamily");
+	font_size = tp_asv_get_int32 (priv->data->info, "DefaultFontSize", NULL);
+	webkit_settings = webkit_web_settings_new ();
+	if (font_family) {
+		g_object_set (G_OBJECT (webkit_settings), "default-font-family", font_family, NULL);
+	}
+	if (font_size) {
+		g_object_set (G_OBJECT (webkit_settings), "default-font-size", font_size, NULL);
+	}
+	webkit_web_view_set_settings (WEBKIT_WEB_VIEW (object), webkit_settings);
+
+	/* Load template */
+	basedir_uri = g_strconcat ("file://", priv->data->basedir, NULL);
+	webkit_web_view_load_html_string (WEBKIT_WEB_VIEW (object),
+					  priv->data->template_html,
+					  basedir_uri);
+
+	g_object_unref (webkit_settings);
+	g_free (basedir_uri);
 }
 
 static void
@@ -728,8 +882,8 @@ theme_adium_get_property (GObject    *object,
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (object);
 
 	switch (param_id) {
-	case PROP_PATH:
-		g_value_set_string (value, priv->path);
+	case PROP_ADIUM_DATA:
+		g_value_set_boxed (value, priv->data);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -746,9 +900,9 @@ theme_adium_set_property (GObject      *object,
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (object);
 
 	switch (param_id) {
-	case PROP_PATH:
-		g_free (priv->path);
-		priv->path = g_value_dup_string (value);
+	case PROP_ADIUM_DATA:
+		g_assert (priv->data == NULL);
+		priv->data = g_value_dup_boxed (value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -760,7 +914,7 @@ static void
 empathy_theme_adium_class_init (EmpathyThemeAdiumClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	
+
 	object_class->finalize = theme_adium_finalize;
 	object_class->dispose = theme_adium_dispose;
 	object_class->constructed = theme_adium_constructed;
@@ -768,13 +922,14 @@ empathy_theme_adium_class_init (EmpathyThemeAdiumClass *klass)
 	object_class->set_property = theme_adium_set_property;
 
 	g_object_class_install_property (object_class,
-					 PROP_PATH,
-					 g_param_spec_string ("path",
-							      "The theme path",
-							      "Path to the adium theme",
-							      g_get_home_dir (),
+					 PROP_ADIUM_DATA,
+					 g_param_spec_boxed ("adium-data",
+							     "The theme data",
+							     "Data for the adium theme",
+							      EMPATHY_TYPE_ADIUM_DATA,
 							      G_PARAM_CONSTRUCT_ONLY |
-							      G_PARAM_READWRITE));
+							      G_PARAM_READWRITE |
+							      G_PARAM_STATIC_STRINGS));
 
 
 	g_type_class_add_private (object_class, sizeof (EmpathyThemeAdiumPriv));
@@ -786,7 +941,7 @@ empathy_theme_adium_init (EmpathyThemeAdium *theme)
 	EmpathyThemeAdiumPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (theme,
 		EMPATHY_TYPE_THEME_ADIUM, EmpathyThemeAdiumPriv);
 
-	theme->priv = priv;	
+	theme->priv = priv;
 
 	priv->smiley_manager = empathy_smiley_manager_dup_singleton ();
 
@@ -799,24 +954,36 @@ empathy_theme_adium_init (EmpathyThemeAdium *theme)
 	g_signal_connect (theme, "populate-popup",
 			  G_CALLBACK (theme_adium_populate_popup_cb),
 			  NULL);
+	g_signal_connect (theme, "hovering-over-link",
+			  G_CALLBACK (theme_adium_hovering_over_link_cb),
+			  NULL);
 }
 
 EmpathyThemeAdium *
-empathy_theme_adium_new (const gchar *path)
+empathy_theme_adium_new (EmpathyAdiumData *data)
 {
-	g_return_val_if_fail (empathy_theme_adium_is_valid (path), NULL);
+	g_return_val_if_fail (data != NULL, NULL);
 
 	return g_object_new (EMPATHY_TYPE_THEME_ADIUM,
-			     "path", path,
+			     "adium-data", data,
 			     NULL);
 }
 
 gboolean
-empathy_theme_adium_is_valid (const gchar *path)
+empathy_adium_path_is_valid (const gchar *path)
 {
 	gboolean ret;
 	gchar   *file;
 
+	/* The theme is not valid if there is no Info.plist */
+	file = g_build_filename (path, "Contents", "Info.plist",
+				 NULL);
+	ret = g_file_test (file, G_FILE_TEST_EXISTS);
+	g_free (file);
+
+	if (ret == FALSE)
+		return ret;
+
 	/* We ship a default Template.html as fallback if there is any problem
 	 * with the one inside the theme. The only other required file is
 	 * Content.html for incoming messages (outgoing fallback to use
@@ -829,3 +996,251 @@ empathy_theme_adium_is_valid (const gchar *path)
 	return ret;
 }
 
+GHashTable *
+empathy_adium_info_new (const gchar *path)
+{
+	gchar *file;
+	GValue *value;
+	GHashTable *info = NULL;
+
+	g_return_val_if_fail (empathy_adium_path_is_valid (path), NULL);
+
+	file = g_build_filename (path, "Contents", "Info.plist", NULL);
+	value = empathy_plist_parse_from_file (file);
+	g_free (file);
+
+	if (value == NULL)
+		return NULL;
+
+	info = g_value_dup_boxed (value);
+	tp_g_value_slice_free (value);
+
+	/* Insert the theme's path into the hash table,
+	 * keys have to be dupped */
+	tp_asv_set_string (info, g_strdup ("path"), path);
+
+	return info;
+}
+
+GType
+empathy_adium_data_get_type (void)
+{
+  static GType type_id = 0;
+
+  if (!type_id)
+    {
+      type_id = g_boxed_type_register_static ("EmpathyAdiumData",
+          (GBoxedCopyFunc) empathy_adium_data_ref,
+          (GBoxedFreeFunc) empathy_adium_data_unref);
+    }
+
+  return type_id;
+}
+
+EmpathyAdiumData  *
+empathy_adium_data_new_with_info (const gchar *path, GHashTable *info)
+{
+	EmpathyAdiumData *data;
+	gchar            *file;
+	gchar            *template_html = NULL;
+	gsize             template_len;
+	gchar            *footer_html = NULL;
+	gsize             footer_len;
+	GString          *string;
+	gchar           **strv = NULL;
+	gchar            *css_path;
+	guint             len = 0;
+	guint             i = 0;
+
+	g_return_val_if_fail (empathy_adium_path_is_valid (path), NULL);
+
+	data = g_slice_new0 (EmpathyAdiumData);
+	data->ref_count = 1;
+	data->path = g_strdup (path);
+	data->basedir = g_strconcat (path, G_DIR_SEPARATOR_S "Contents"
+		G_DIR_SEPARATOR_S "Resources" G_DIR_SEPARATOR_S, NULL);
+	data->info = g_hash_table_ref (info);
+
+	/* Load html files */
+	file = g_build_filename (data->basedir, "Incoming", "Content.html", NULL);
+	g_file_get_contents (file, &data->in_content_html, &data->in_content_len, NULL);
+	g_free (file);
+
+	file = g_build_filename (data->basedir, "Incoming", "NextContent.html", NULL);
+	g_file_get_contents (file, &data->in_nextcontent_html, &data->in_nextcontent_len, NULL);
+	g_free (file);
+
+	file = g_build_filename (data->basedir, "Incoming", "Context.html", NULL);
+	g_file_get_contents (file, &data->in_context_html, &data->in_context_len, NULL);
+	g_free (file);
+
+	file = g_build_filename (data->basedir, "Incoming", "NextContext.html", NULL);
+	g_file_get_contents (file, &data->in_nextcontext_html, &data->in_nextcontext_len, NULL);
+	g_free (file);
+
+	file = g_build_filename (data->basedir, "Outgoing", "Content.html", NULL);
+	g_file_get_contents (file, &data->out_content_html, &data->out_content_len, NULL);
+	g_free (file);
+
+	file = g_build_filename (data->basedir, "Outgoing", "NextContent.html", NULL);
+	g_file_get_contents (file, &data->out_nextcontent_html, &data->out_nextcontent_len, NULL);
+	g_free (file);
+
+	file = g_build_filename (data->basedir, "Outgoing", "Context.html", NULL);
+	g_file_get_contents (file, &data->out_context_html, &data->out_context_len, NULL);
+	g_free (file);
+
+	file = g_build_filename (data->basedir, "Outgoing", "NextContext.html", NULL);
+	g_file_get_contents (file, &data->out_nextcontext_html, &data->out_nextcontext_len, NULL);
+	g_free (file);
+
+	file = g_build_filename (data->basedir, "Status.html", NULL);
+	g_file_get_contents (file, &data->status_html, &data->status_len, NULL);
+	g_free (file);
+
+	file = g_build_filename (data->basedir, "Footer.html", NULL);
+	g_file_get_contents (file, &footer_html, &footer_len, NULL);
+	g_free (file);
+
+	file = g_build_filename (data->basedir, "Incoming", "buddy_icon.png", NULL);
+	if (g_file_test (file, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
+		data->default_incoming_avatar_filename = file;
+	} else {
+		g_free (file);
+	}
+
+	file = g_build_filename (data->basedir, "Outgoing", "buddy_icon.png", NULL);
+	if (g_file_test (file, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
+		data->default_outgoing_avatar_filename = file;
+	} else {
+		g_free (file);
+	}
+
+	css_path = g_build_filename (data->basedir, "main.css", NULL);
+
+	/* There is 2 formats for Template.html: The old one has 4 parameters,
+	 * the new one has 5 parameters. */
+	file = g_build_filename (data->basedir, "Template.html", NULL);
+	if (g_file_get_contents (file, &template_html, &template_len, NULL)) {
+		strv = g_strsplit (template_html, "%@", -1);
+		len = g_strv_length (strv);
+	}
+	g_free (file);
+
+	if (len != 5 && len != 6) {
+		/* Either the theme has no template or it don't have the good
+		 * number of parameters. Fallback to use our own template. */
+		g_free (template_html);
+		g_strfreev (strv);
+
+		file = empathy_file_lookup ("Template.html", "data");
+		g_file_get_contents (file, &template_html, &template_len, NULL);
+		g_free (file);
+		strv = g_strsplit (template_html, "%@", -1);
+		len = g_strv_length (strv);
+	}
+
+	/* Replace %@ with the needed information in the template html. */
+	string = g_string_sized_new (template_len);
+	g_string_append (string, strv[i++]);
+	g_string_append (string, data->basedir);
+	g_string_append (string, strv[i++]);
+	if (len == 6) {
+		const gchar *variant;
+
+		/* We include main.css by default */
+		g_string_append_printf (string, "@import url(\"%s\");", css_path);
+		g_string_append (string, strv[i++]);
+		variant = tp_asv_get_string (data->info, "DefaultVariant");
+		if (variant) {
+			g_string_append (string, "Variants/");
+			g_string_append (string, variant);
+			g_string_append (string, ".css");
+		}
+	} else {
+		/* FIXME: We should set main.css OR the variant css */
+		g_string_append (string, css_path);
+	}
+	g_string_append (string, strv[i++]);
+	g_string_append (string, ""); /* We don't want header */
+	g_string_append (string, strv[i++]);
+	/* FIXME: We should replace adium %macros% in footer */
+	if (footer_html) {
+		g_string_append (string, footer_html);
+	}
+	g_string_append (string, strv[i++]);
+	data->template_html = g_string_free (string, FALSE);
+
+	g_free (footer_html);
+	g_free (template_html);
+	g_free (css_path);
+	g_strfreev (strv);
+
+	return data;
+}
+
+EmpathyAdiumData  *
+empathy_adium_data_new (const gchar *path)
+{
+	EmpathyAdiumData *data;
+	GHashTable *info;
+
+	info = empathy_adium_info_new (path);
+	data = empathy_adium_data_new_with_info (path, info);
+	g_hash_table_unref (info);
+
+	return data;
+}
+
+EmpathyAdiumData  *
+empathy_adium_data_ref (EmpathyAdiumData *data)
+{
+	g_return_val_if_fail (data != NULL, NULL);
+
+	g_atomic_int_inc (&data->ref_count);
+
+	return data;
+}
+
+void
+empathy_adium_data_unref (EmpathyAdiumData *data)
+{
+	g_return_if_fail (data != NULL);
+
+	if (g_atomic_int_dec_and_test (&data->ref_count)) {
+		g_free (data->path);
+		g_free (data->basedir);
+		g_free (data->template_html);
+		g_free (data->in_content_html);
+		g_free (data->in_nextcontent_html);
+		g_free (data->in_context_html);
+		g_free (data->in_nextcontext_html);
+		g_free (data->out_content_html);
+		g_free (data->out_nextcontent_html);
+		g_free (data->out_context_html);
+		g_free (data->out_nextcontext_html);
+		g_free (data->default_avatar_filename);
+		g_free (data->default_incoming_avatar_filename);
+		g_free (data->default_outgoing_avatar_filename);
+		g_free (data->status_html);
+		g_hash_table_unref (data->info);
+		g_slice_free (EmpathyAdiumData, data);
+	}
+}
+
+GHashTable *
+empathy_adium_data_get_info (EmpathyAdiumData *data)
+{
+	g_return_val_if_fail (data != NULL, NULL);
+
+	return data->info;
+}
+
+const gchar *
+empathy_adium_data_get_path (EmpathyAdiumData *data)
+{
+	g_return_val_if_fail (data != NULL, NULL);
+
+	return data->path;
+}
+
