@@ -34,33 +34,16 @@
 
 #include <gtk/gtk.h>
 
-#include <canberra-gtk.h>
-
-#include <libmissioncontrol/mc-account.h>
 #include <libmissioncontrol/mc-profile.h>
 
-
 #include <libempathy/empathy-contact.h>
+#include <libempathy/empathy-account.h>
+#include <libempathy/empathy-ft-handler.h>
 
 #include "empathy-chat-view.h"
 
 G_BEGIN_DECLS
 
-/* NOTE: Keep this sync with sound_entries in empathy-ui-utils.c */
-typedef enum {
-	EMPATHY_SOUND_MESSAGE_INCOMING = 0,
-	EMPATHY_SOUND_MESSAGE_OUTGOING,
-	EMPATHY_SOUND_CONVERSATION_NEW,
-	EMPATHY_SOUND_CONTACT_CONNECTED,
-	EMPATHY_SOUND_CONTACT_DISCONNECTED,
-	EMPATHY_SOUND_ACCOUNT_CONNECTED,
-	EMPATHY_SOUND_ACCOUNT_DISCONNECTED,
-	EMPATHY_SOUND_PHONE_INCOMING,
-	EMPATHY_SOUND_PHONE_OUTGOING,
-	EMPATHY_SOUND_PHONE_HANGUP,
-	LAST_EMPATHY_SOUND,
-} EmpathySound;
-
 void            empathy_gtk_init                        (void);
 GRegex *        empathy_uri_regex_dup_singleton         (void);
 
@@ -76,8 +59,8 @@ GtkWidget     *empathy_builder_unref_and_keep_widget    (GtkBuilder       *gui,
 							 GtkWidget        *root);
 
 /* Pixbufs */
-const gchar * empathy_icon_name_from_account            (McAccount        *account);
-const gchar * empathy_icon_name_for_presence            (McPresence        presence);
+const gchar * empathy_icon_name_from_account            (EmpathyAccount   *account);
+const gchar * empathy_icon_name_for_presence            (TpConnectionPresenceType  presence);
 const gchar * empathy_icon_name_for_contact             (EmpathyContact   *contact);
 GdkPixbuf *   empathy_pixbuf_from_data                  (gchar            *data,
 							 gsize             data_size);
@@ -90,12 +73,15 @@ GdkPixbuf *   empathy_pixbuf_from_avatar_scaled         (EmpathyAvatar    *avata
 GdkPixbuf *   empathy_pixbuf_avatar_from_contact_scaled (EmpathyContact   *contact,
 							 gint              width,
 							 gint              height);
-GdkPixbuf * empathy_pixbuf_scale_down_if_necessary      (GdkPixbuf        *pixbuf,
+GdkPixbuf *   empathy_pixbuf_scale_down_if_necessary    (GdkPixbuf        *pixbuf,
 							 gint              max_size);
-GdkPixbuf * empathy_pixbuf_from_icon_name               (const gchar      *icon_name,
+GdkPixbuf *   empathy_pixbuf_from_icon_name             (const gchar      *icon_name,
 							 GtkIconSize       icon_size);
-GdkPixbuf * empathy_pixbuf_from_icon_name_sized         (const gchar      *icon_name,
+GdkPixbuf *   empathy_pixbuf_from_icon_name_sized       (const gchar      *icon_name,
 							 gint              size);
+gchar *       empathy_filename_from_icon_name           (const gchar      *icon_name,
+							 GtkIconSize       icon_size);
+
 /* Text view */
 gboolean    empathy_text_iter_forward_search            (const GtkTextIter*iter,
 							 const gchar      *str,
@@ -124,15 +110,7 @@ GtkWidget * empathy_link_button_new                     (const gchar      *url,
 							 const gchar      *title);
 
 void        empathy_send_file_with_file_chooser         (EmpathyContact   *contact);
-
-/* Sounds */
-void        empathy_sound_play                          (GtkWidget        *widget,
-							 EmpathySound      sound_id);
-gboolean    empathy_sound_play_full                     (GtkWidget *widget,
-							 EmpathySound sound_id,
-							 ca_finish_callback_t callback,
-							 gpointer user_data);
-void        empathy_sound_stop                          (EmpathySound sound_id);
+void        empathy_receive_file_with_file_chooser      (EmpathyFTHandler *handler);
 
 G_END_DECLS
 
