@@ -14,7 +14,7 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- * 
+ *
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  *          Sjoerd Simons <sjoerd.simons@collabora.co.uk>
  */
@@ -26,19 +26,22 @@
 
 #include <telepathy-glib/util.h>
 
+#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-dispatcher.h>
-#include <libempathy/empathy-contact-factory.h>
+#include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-tp-chat.h>
 #include <libempathy/empathy-tp-call.h>
+#include <libempathy/empathy-tp-file.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-call-factory.h>
 
 #include <extensions/extensions.h>
 
+#include <libempathy-gtk/empathy-conf.h>
 #include <libempathy-gtk/empathy-images.h>
 #include <libempathy-gtk/empathy-contact-dialogs.h>
-#include <libempathy-gtk/empathy-ui-utils.h>
+#include <libempathy-gtk/empathy-sound.h>
 
 #include "empathy-event-manager.h"
 #include "empathy-main-window.h"
@@ -49,6 +52,11 @@
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyEventManager)
 
+#define NOTIFICATION_TIMEOUT 2 /* seconds */
+
+/* The time interval in milliseconds between 2 incoming rings */
+#define MS_BETWEEN_RING 500
+
 typedef struct {
   EmpathyEventManager *manager;
   EmpathyDispatchOperation *operation;
@@ -59,8 +67,9 @@ typedef struct {
   EmpathyContact *contact;
   /* Tube dispatcher if applicable */
   EmpathyTubeDispatch *tube_dispatch;
-  /* option signal handler */
+  /* option signal handler and it's instance */
   gulong handler;
+  GObject *handler_instance;
   /* optional accept widget */
   GtkWidget *dialog;
 } EventManagerApproval;
@@ -72,8 +81,6 @@ typedef struct {
   /* Ongoing approvals */
   GSList *approvals;
 
-  /* voip ringing sound */
-  guint voip_timeout;
   gint ringing;
 } EmpathyEventManagerPriv;
 
@@ -124,6 +131,10 @@ event_manager_approval_free (EventManagerApproval *approval)
     approval->invalidated_handler);
   g_object_unref (approval->operation);
 
+  if (approval->handler != 0)
+    g_signal_handler_disconnect (approval->handler_instance,
+      approval->handler);
+
   if (approval->contact != NULL)
     g_object_unref (approval->contact);
 
@@ -155,81 +166,6 @@ event_free (EventPriv *event)
   g_slice_free (EventPriv, event);
 }
 
-static void event_manager_ringing_finished_cb (ca_context *c, guint id,
-  int error_code, gpointer user_data);
-
-static gboolean
-event_manager_ringing_timeout_cb (gpointer data)
-{
-  EmpathyEventManager *manager = EMPATHY_EVENT_MANAGER (data);
-  EmpathyEventManagerPriv *priv = GET_PRIV (manager);
-
-  priv->voip_timeout = 0;
-
-  empathy_sound_play_full (empathy_main_window_get (),
-      EMPATHY_SOUND_PHONE_INCOMING, event_manager_ringing_finished_cb,
-      manager);
-
-  return FALSE;
-}
-
-static gboolean
-event_manager_ringing_idle_cb (gpointer data)
-{
-  EmpathyEventManager *manager = EMPATHY_EVENT_MANAGER (data);
-  EmpathyEventManagerPriv *priv = GET_PRIV (manager);
-
-  if (priv->ringing > 0)
-    priv->voip_timeout = g_timeout_add (500, event_manager_ringing_timeout_cb,
-      data);
-
-  return FALSE;
-}
-
-static void
-event_manager_ringing_finished_cb (ca_context *c, guint id, int error_code,
-  gpointer user_data)
-{
-  if (error_code == CA_ERROR_CANCELED)
-    return;
-
-  g_idle_add (event_manager_ringing_idle_cb, user_data);
-}
-
-static void
-event_manager_start_ringing (EmpathyEventManager *manager)
-{
-  EmpathyEventManagerPriv *priv = GET_PRIV (manager);
-
-  priv->ringing++;
-
-  if (priv->ringing == 1)
-    {
-      empathy_sound_play_full (empathy_main_window_get (),
-        EMPATHY_SOUND_PHONE_INCOMING, event_manager_ringing_finished_cb,
-        manager);
-    }
-}
-
-static void
-event_manager_stop_ringing (EmpathyEventManager *manager)
-{
-  EmpathyEventManagerPriv *priv = GET_PRIV (manager);
-
-  priv->ringing--;
-
-  if (priv->ringing > 0)
-    return;
-
-  empathy_sound_stop (EMPATHY_SOUND_PHONE_INCOMING);
-
-  if (priv->voip_timeout != 0)
-    {
-      g_source_remove (priv->voip_timeout);
-      priv->voip_timeout = 0;
-    }
-}
-
 static void
 event_remove (EventPriv *event)
 {
@@ -241,6 +177,13 @@ event_remove (EventPriv *event)
   event_free (event);
 }
 
+static gboolean
+autoremove_event_timeout_cb (EventPriv *event)
+{
+  event_remove (event);
+  return FALSE;
+}
+
 static void
 event_manager_add (EmpathyEventManager *manager, EmpathyContact *contact,
   const gchar *icon_name, const gchar *header, const gchar *message,
@@ -254,6 +197,7 @@ event_manager_add (EmpathyEventManager *manager, EmpathyContact *contact,
   event->public.icon_name = g_strdup (icon_name);
   event->public.header = g_strdup (header);
   event->public.message = g_strdup (message);
+  event->public.must_ack = (func != NULL);
   event->inhibit = FALSE;
   event->func = func;
   event->user_data = user_data;
@@ -263,6 +207,12 @@ event_manager_add (EmpathyEventManager *manager, EmpathyContact *contact,
   DEBUG ("Adding event %p", event);
   priv->events = g_slist_prepend (priv->events, event);
   g_signal_emit (event->manager, signals[EVENT_ADDED], 0, event);
+
+  if (!event->public.must_ack)
+    {
+      g_timeout_add_seconds (NOTIFICATION_TIMEOUT,
+        (GSourceFunc) autoremove_event_timeout_cb, event);
+    }
 }
 
 static void
@@ -280,7 +230,7 @@ event_text_channel_process_func (EventPriv *event)
     {
       tp_chat = EMPATHY_TP_CHAT
         (empathy_dispatch_operation_get_channel_wrapper (event->approval->operation));
-  
+
       g_signal_handler_disconnect (tp_chat, event->approval->handler);
       event->approval->handler = 0;
     }
@@ -367,8 +317,7 @@ event_channel_process_voip_func (EventPriv *event)
       return;
     }
 
-  dialog = gtk_message_dialog_new (GTK_WINDOW (empathy_main_window_get()),
-      GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+  dialog = gtk_message_dialog_new (NULL, 0,
       GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, _("Incoming call"));
   gtk_message_dialog_format_secondary_text (
     GTK_MESSAGE_DIALOG (dialog),
@@ -404,7 +353,7 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
   EmpathyMessage *message, EventManagerApproval *approval)
 {
   EmpathyContact  *sender;
-  gchar           *header;
+  const gchar     *header;
   const gchar     *msg;
   TpChannel       *channel;
   EventPriv       *event;
@@ -421,8 +370,7 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
     }
 
   sender = empathy_message_get_sender (message);
-  header = g_strdup_printf (_("New message from %s"),
-                            empathy_contact_get_name (sender));
+  header = empathy_contact_get_name (sender);
   msg = empathy_message_get_body (message);
 
   channel = empathy_tp_chat_get_channel (tp_chat);
@@ -433,7 +381,6 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
     event_manager_add (approval->manager, sender, EMPATHY_IMAGE_NEW_MESSAGE, header,
       msg, approval, event_text_channel_process_func, NULL);
 
-  g_free (header);
   empathy_sound_play (empathy_main_window_get (),
     EMPATHY_SOUND_CONVERSATION_NEW);
 }
@@ -452,7 +399,9 @@ event_manager_approval_done (EventManagerApproval *approval)
           approval->operation);
       if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
         {
-          event_manager_stop_ringing (approval->manager);
+          priv->ringing--;
+          if (priv->ringing == 0)
+            empathy_sound_stop (EMPATHY_SOUND_PHONE_INCOMING);
         }
     }
 
@@ -495,36 +444,24 @@ event_manager_operation_invalidated_cb (EmpathyDispatchOperation *operation,
 }
 
 static void
-event_manager_media_channel_got_name_cb (EmpathyContact *contact,
-  const GError *error, gpointer user_data, GObject *object)
+event_manager_media_channel_got_contact (EventManagerApproval *approval)
 {
-  EventManagerApproval *approval = user_data;
+  EmpathyEventManagerPriv *priv = GET_PRIV (approval->manager);
   gchar *header;
 
-  if (error != NULL)
-    {
-      /* FIXME just returning assuming the operation will be invalidated as
-       * well */
-      return;
-    }
-
   header = g_strdup_printf (_("Incoming call from %s"),
-    empathy_contact_get_name (contact));
+    empathy_contact_get_name (approval->contact));
 
   event_manager_add (approval->manager,
     approval->contact, EMPATHY_IMAGE_VOIP, header, NULL,
     approval, event_channel_process_voip_func, NULL);
 
   g_free (header);
-  event_manager_start_ringing (approval->manager);
-}
 
-static void
-event_manager_media_channel_got_contact (EventManagerApproval *approval)
-{
-  empathy_contact_call_when_ready (approval->contact,
-     EMPATHY_CONTACT_READY_NAME, event_manager_media_channel_got_name_cb,
-        approval, NULL, G_OBJECT (approval->manager));
+  priv->ringing++;
+  if (priv->ringing == 1)
+    empathy_sound_start_playing (empathy_main_window_get (),
+        EMPATHY_SOUND_PHONE_INCOMING, MS_BETWEEN_RING);
 }
 
 static void
@@ -584,7 +521,7 @@ static void
 event_manager_tube_dispatch_ability_cb (GObject *object,
    GParamSpec *spec, gpointer user_data)
 {
-  EventManagerApproval *approval = (EventManagerApproval *)user_data;
+  EventManagerApproval *approval = (EventManagerApproval *) user_data;
   EmpathyTubeDispatchAbility dispatchability;
 
   dispatchability =
@@ -599,29 +536,34 @@ event_manager_tube_dispatch_ability_cb (GObject *object,
 }
 
 static void
-event_manager_tube_got_contact_name_cb (EmpathyContact *contact,
-  const GError *error, gpointer user_data, GObject *object)
+event_manager_tube_got_contact_cb (EmpathyTpContactFactory *factory,
+                                   EmpathyContact *contact,
+                                   const GError *error,
+                                   gpointer user_data,
+                                   GObject *object)
 {
-  EventManagerApproval *approval = (EventManagerApproval *)user_data;
+  EventManagerApproval *approval = (EventManagerApproval *) user_data;
   EmpathyTubeDispatchAbility dispatchability;
 
   if (error != NULL)
     {
-      /* FIXME?, we assume that the operation gets invalidated as well (if it
-       * didn't already */
-       return;
+      /* FIXME: We should probably still display the event */
+      DEBUG ("Error: %s", error->message);
+      return;
     }
 
+  approval->contact = g_object_ref (contact);
+
   dispatchability = empathy_tube_dispatch_is_dispatchable
     (approval->tube_dispatch);
 
-
   switch (dispatchability)
     {
       case EMPATHY_TUBE_DISPATCHABILITY_UNKNOWN:
         approval->handler = g_signal_connect (approval->tube_dispatch,
           "notify::dispatchability",
           G_CALLBACK (event_manager_tube_dispatch_ability_cb), approval);
+        approval->handler_instance = G_OBJECT (approval->tube_dispatch);
         break;
       case EMPATHY_TUBE_DISPATCHABILITY_POSSIBLE:
         /* fallthrough */
@@ -631,6 +573,159 @@ event_manager_tube_got_contact_name_cb (EmpathyContact *contact,
     }
 }
 
+static void
+invite_dialog_response_cb (GtkDialog *dialog,
+                           gint response,
+                           EventManagerApproval *approval)
+{
+  EmpathyTpChat *tp_chat;
+  TpChannel *channel;
+  TpHandle self_handle;
+  GArray *members;
+
+  gtk_widget_destroy (GTK_WIDGET (approval->dialog));
+  approval->dialog = NULL;
+
+  tp_chat = EMPATHY_TP_CHAT (empathy_dispatch_operation_get_channel_wrapper (
+        approval->operation));
+
+  if (response != GTK_RESPONSE_OK)
+    {
+      /* close channel */
+      DEBUG ("Muc invitation rejected");
+
+      if (empathy_dispatch_operation_claim (approval->operation))
+        empathy_tp_chat_close (tp_chat);
+      return;
+    }
+
+  DEBUG ("Muc invitation accepted");
+
+  /* join the room */
+  channel = empathy_tp_chat_get_channel (tp_chat);
+
+  self_handle = tp_channel_group_get_self_handle (channel);
+  members = g_array_sized_new (FALSE, FALSE, sizeof (TpHandle), 1);
+  g_array_append_val (members, self_handle);
+
+  tp_cli_channel_interface_group_call_add_members (channel, -1, members,
+      "", NULL, NULL, NULL, NULL);
+
+  empathy_dispatch_operation_approve (approval->operation);
+
+  g_array_free (members, TRUE);
+}
+
+static void
+event_room_channel_process_func (EventPriv *event)
+{
+  GtkWidget *dialog, *button, *image;
+  TpChannel *channel = empathy_dispatch_operation_get_channel (
+      event->approval->operation);
+
+  if (event->approval->dialog != NULL)
+    {
+      gtk_window_present (GTK_WINDOW (event->approval->dialog));
+      return;
+    }
+
+  /* create dialog */
+  dialog = gtk_message_dialog_new (NULL, 0,
+      GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, _("Room invitation"));
+
+  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+      _("%s is inviting you to join %s"),
+      empathy_contact_get_name (event->approval->contact),
+      tp_channel_get_identifier (channel));
+
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+      GTK_RESPONSE_OK);
+
+  button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+      _("_Decline"), GTK_RESPONSE_CANCEL);
+  image = gtk_image_new_from_icon_name (GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON);
+  gtk_button_set_image (GTK_BUTTON (button), image);
+
+  button = gtk_dialog_add_button (GTK_DIALOG (dialog),
+      _("_Join"), GTK_RESPONSE_OK);
+  image = gtk_image_new_from_icon_name (GTK_STOCK_APPLY, GTK_ICON_SIZE_BUTTON);
+  gtk_button_set_image (GTK_BUTTON (button), image);
+
+  g_signal_connect (dialog, "response",
+      G_CALLBACK (invite_dialog_response_cb), event->approval);
+
+  gtk_widget_show (dialog);
+
+  event->approval->dialog = dialog;
+}
+
+static void
+event_manager_muc_invite_got_contact_cb (EmpathyTpContactFactory *factory,
+                                         EmpathyContact *contact,
+                                         const GError *error,
+                                         gpointer user_data,
+                                         GObject *object)
+{
+  EventManagerApproval *approval = (EventManagerApproval *) user_data;
+  TpChannel *channel;
+  const gchar *invite_msg;
+  gchar *msg;
+  TpHandle self_handle;
+
+  if (error != NULL)
+    {
+      /* FIXME: We should probably still display the event */
+      DEBUG ("Error: %s", error->message);
+      return;
+    }
+
+  approval->contact = g_object_ref (contact);
+  channel = empathy_dispatch_operation_get_channel (approval->operation);
+
+  self_handle = tp_channel_group_get_self_handle (channel);
+  tp_channel_group_get_local_pending_info (channel, self_handle, NULL, NULL,
+      &invite_msg);
+
+  msg = g_strdup_printf (_("%s invited you to join %s"),
+      empathy_contact_get_name (approval->contact),
+      tp_channel_get_identifier (channel));
+
+  event_manager_add (approval->manager,
+    approval->contact, EMPATHY_IMAGE_GROUP_MESSAGE, msg, invite_msg,
+    approval, event_room_channel_process_func, NULL);
+
+  empathy_sound_play (empathy_main_window_get (),
+    EMPATHY_SOUND_CONVERSATION_NEW);
+
+  g_free (msg);
+}
+
+static void
+event_manager_ft_got_contact_cb (EmpathyTpContactFactory *factory,
+                                 EmpathyContact *contact,
+                                 const GError *error,
+                                 gpointer user_data,
+                                 GObject *object)
+{
+  EventManagerApproval *approval = (EventManagerApproval *) user_data;
+  char *header;
+
+  approval->contact = contact;
+
+  header = g_strdup_printf (_("Incoming file transfer from %s"),
+                            empathy_contact_get_name (approval->contact));
+
+  event_manager_add (approval->manager, approval->contact,
+      EMPATHY_IMAGE_DOCUMENT_SEND, header, NULL, approval,
+      event_channel_process_func, NULL);
+
+  /* FIXME better sound for incoming file transfers ?*/
+  empathy_sound_play (empathy_main_window_get (),
+                      EMPATHY_SOUND_CONVERSATION_NEW);
+
+  g_free (header);
+}
+
 static void
 event_manager_approve_channel_cb (EmpathyDispatcher *dispatcher,
   EmpathyDispatchOperation  *operation, EmpathyEventManager *manager)
@@ -658,10 +753,43 @@ event_manager_approve_channel_cb (EmpathyDispatcher *dispatcher,
       EmpathyTpChat *tp_chat =
         EMPATHY_TP_CHAT (
           empathy_dispatch_operation_get_channel_wrapper (operation));
+      TpChannel *channel = empathy_tp_chat_get_channel (tp_chat);
 
+      if (tp_proxy_has_interface (channel, TP_IFACE_CHANNEL_INTERFACE_GROUP))
+        {
+          /* Are we in local-pending ? */
+          TpHandle self_handle, inviter;
+
+          self_handle = tp_channel_group_get_self_handle (channel);
+
+          if (self_handle != 0 && tp_channel_group_get_local_pending_info (
+                channel, self_handle, &inviter, NULL, NULL))
+            {
+              /* We are invited to a room */
+              EmpathyTpContactFactory *factory;
+              TpConnection *connection;
+
+              DEBUG ("Have been invited to %s. Ask user if he wants to accept",
+                  tp_channel_get_identifier (channel));
+
+              connection = empathy_tp_chat_get_connection (tp_chat);
+              factory = empathy_tp_contact_factory_dup_singleton (connection);
+
+              empathy_tp_contact_factory_get_from_handle (factory,
+                  inviter, event_manager_muc_invite_got_contact_cb,
+                  approval, NULL, G_OBJECT (manager));
+
+              g_object_unref (factory);
+              return;
+            }
+
+          /* if we are not invited, let's wait for the first message */
+        }
+
+      /* 1-1 text channel, wait for the first message */
       approval->handler = g_signal_connect (tp_chat, "message-received",
         G_CALLBACK (event_manager_chat_message_received_cb), approval);
-
+      approval->handler_instance = G_OBJECT (tp_chat);
     }
   else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA))
     {
@@ -684,79 +812,48 @@ event_manager_approve_channel_cb (EmpathyDispatcher *dispatcher,
         }
 
     }
-  else if (!tp_strdiff (channel_type, EMP_IFACE_CHANNEL_TYPE_FILE_TRANSFER))
+  else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER))
     {
-      EmpathyContact        *contact;
-      gchar                 *header;
-      TpHandle               handle;
-      McAccount             *account;
-      EmpathyContactFactory *factory;
-      TpChannel *channel = empathy_dispatch_operation_get_channel (operation);
-
-      factory = empathy_contact_factory_dup_singleton ();
-      handle = tp_channel_get_handle (channel, NULL);
-      account = empathy_channel_get_account (channel);
-
-      contact = empathy_contact_factory_get_from_handle (factory, account,
-        handle);
-
-      empathy_contact_run_until_ready (contact,
-        EMPATHY_CONTACT_READY_NAME, NULL);
-
-      header = g_strdup_printf (_("Incoming file transfer from %s"),
-        empathy_contact_get_name (contact));
-
-      event_manager_add (manager, contact, EMPATHY_IMAGE_DOCUMENT_SEND,
-        header, NULL, approval, event_channel_process_func, NULL);
-
-      /* FIXME better sound for incoming file transfers ?*/
-      empathy_sound_play (empathy_main_window_get (),
-        EMPATHY_SOUND_CONVERSATION_NEW);
-
-      g_object_unref (factory);
-      g_object_unref (account);
-      g_free (header);
-    }
-  else if (!tp_strdiff (channel_type, EMP_IFACE_CHANNEL_TYPE_STREAM_TUBE) ||
-      !tp_strdiff (channel_type, EMP_IFACE_CHANNEL_TYPE_DBUS_TUBE))
-    {
-      EmpathyContact        *contact;
-      TpHandle               handle;
-      TpHandleType           handle_type;
-      McAccount             *account;
-      EmpathyContactFactory *factory;
-      EmpathyTubeDispatch *tube_dispatch;
       TpChannel *channel;
+      TpConnection *connection;
+      TpHandle handle;
+      EmpathyTpContactFactory *factory;
 
       channel = empathy_dispatch_operation_get_channel (operation);
+      handle = tp_channel_get_handle (channel, NULL);
 
+      connection = tp_channel_borrow_connection (channel);
+      factory = empathy_tp_contact_factory_dup_singleton (connection);
+      empathy_tp_contact_factory_get_from_handle (factory, handle,
+        event_manager_ft_got_contact_cb, approval, NULL, G_OBJECT (manager));
+
+      g_object_unref (factory);
+    }
+  else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE) ||
+      !tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE))
+    {
+      TpChannel *channel;
+      TpHandle handle;
+      TpHandleType handle_type;
+      TpConnection *connection;
+      EmpathyTpContactFactory *factory;
+
+      channel = empathy_dispatch_operation_get_channel (operation);
       handle = tp_channel_get_handle (channel, &handle_type);
 
       /* Only understand p2p tubes */
       if (handle_type != TP_HANDLE_TYPE_CONTACT)
         return;
 
-      factory = empathy_contact_factory_dup_singleton ();
-      account = empathy_channel_get_account (channel);
-
-      contact = empathy_contact_factory_get_from_handle (factory, account,
-        handle);
-
-      tube_dispatch = empathy_tube_dispatch_new (operation);
-
-      approval->contact = contact;
-      approval->tube_dispatch = tube_dispatch;
-
-      empathy_contact_call_when_ready (contact,
-        EMPATHY_CONTACT_READY_NAME, event_manager_tube_got_contact_name_cb,
-        approval, NULL, G_OBJECT (manager));
-
-      g_object_unref (factory);
-      g_object_unref (account);
+      approval->tube_dispatch = empathy_tube_dispatch_new (operation);
+      connection = tp_channel_borrow_connection (channel);
+      factory = empathy_tp_contact_factory_dup_singleton (connection);
+      empathy_tp_contact_factory_get_from_handle (factory, handle,
+        event_manager_tube_got_contact_cb, approval, NULL, G_OBJECT (manager));
     }
   else
     {
-      DEBUG ("Unknown channel type, ignoring..");
+      DEBUG ("Unknown channel type (%s), ignoring..", channel_type);
     }
 }
 
@@ -784,19 +881,17 @@ event_manager_pendings_changed_cb (EmpathyContactList  *list,
         {
           EventPriv *event = l->data;
 
-      if (event->public.contact == contact &&
-          event->func == event_pending_subscribe_func)
-        {
-          event_remove (event);
-          break;
+          if (event->public.contact == contact &&
+              event->func == event_pending_subscribe_func)
+            {
+              event_remove (event);
+              break;
+            }
         }
-      }
 
       return;
     }
 
-  empathy_contact_run_until_ready (contact, EMPATHY_CONTACT_READY_NAME, NULL);
-
   header = g_strdup_printf (_("Subscription requested by %s"),
     empathy_contact_get_name (contact));
 
@@ -812,6 +907,58 @@ event_manager_pendings_changed_cb (EmpathyContactList  *list,
   g_free (header);
 }
 
+static void
+event_manager_presence_changed_cb (EmpathyContactMonitor *monitor,
+    EmpathyContact *contact,
+    TpConnectionPresenceType current,
+    TpConnectionPresenceType previous,
+    EmpathyEventManager *manager)
+{
+  EmpathyAccount *account;
+  gchar *header = NULL;
+  gboolean preference = FALSE;
+
+  account = empathy_contact_get_account (contact);
+  if (empathy_account_is_just_connected (account))
+    return;
+
+  if (tp_connection_presence_type_cmp_availability (previous,
+     TP_CONNECTION_PRESENCE_TYPE_OFFLINE) > 0)
+    {
+      /* contact was online */
+      empathy_conf_get_bool (empathy_conf_get (),
+                      EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNOUT, &preference);
+      if (preference && tp_connection_presence_type_cmp_availability (current,
+          TP_CONNECTION_PRESENCE_TYPE_OFFLINE) <= 0)
+        {
+          /* someone is logging off */
+          header = g_strdup_printf (_("%s is now offline."),
+            empathy_contact_get_name (contact));
+
+          event_manager_add (manager, contact, GTK_STOCK_DIALOG_INFO, header,
+                             NULL, NULL, NULL, NULL);
+        }
+    }
+  else
+    {
+      /* contact was offline */
+      empathy_conf_get_bool (empathy_conf_get (),
+                      EMPATHY_PREFS_NOTIFICATIONS_CONTACT_SIGNIN, &preference);
+      if (preference && tp_connection_presence_type_cmp_availability (current,
+          TP_CONNECTION_PRESENCE_TYPE_OFFLINE) > 0)
+        {
+          /* someone is logging in */
+          header = g_strdup_printf (_("%s is now online."),
+            empathy_contact_get_name (contact));
+
+          event_manager_add (manager, contact, GTK_STOCK_DIALOG_INFO, header,
+                             NULL, NULL, NULL, NULL);
+        }
+    }
+  g_free (header);
+}
+
+
 static GObject *
 event_manager_constructor (GType type,
 			   guint n_props,
@@ -826,7 +973,7 @@ event_manager_constructor (GType type,
 			(type, n_props, props);
 
 		manager_singleton = EMPATHY_EVENT_MANAGER (retval);
-		g_object_add_weak_pointer (retval, (gpointer *) &manager_singleton);
+		g_object_add_weak_pointer (retval, (gpointer) &manager_singleton);
 	}
 
 	return retval;
@@ -837,6 +984,9 @@ event_manager_finalize (GObject *object)
 {
   EmpathyEventManagerPriv *priv = GET_PRIV (object);
 
+  if (priv->ringing > 0)
+    empathy_sound_stop (EMPATHY_SOUND_PHONE_INCOMING);
+
   g_slist_foreach (priv->events, (GFunc) event_free, NULL);
   g_slist_free (priv->events);
   g_slist_foreach (priv->approvals, (GFunc) event_manager_approval_free, NULL);
@@ -890,6 +1040,12 @@ empathy_event_manager_init (EmpathyEventManager *manager)
 {
   EmpathyEventManagerPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (manager,
     EMPATHY_TYPE_EVENT_MANAGER, EmpathyEventManagerPriv);
+  EmpathyContactMonitor *monitor;
+  EmpathyContactList *list_iface;
+
+  list_iface = EMPATHY_CONTACT_LIST (empathy_contact_manager_dup_singleton ());
+  monitor = empathy_contact_list_get_monitor (list_iface);
+  g_object_unref (list_iface);
 
   manager->priv = priv;
 
@@ -899,6 +1055,8 @@ empathy_event_manager_init (EmpathyEventManager *manager)
     G_CALLBACK (event_manager_approve_channel_cb), manager);
   g_signal_connect (priv->contact_manager, "pendings-changed",
     G_CALLBACK (event_manager_pendings_changed_cb), manager);
+  g_signal_connect (monitor, "contact-presence-changed",
+    G_CALLBACK (event_manager_presence_changed_cb), manager);
 }
 
 EmpathyEventManager *
@@ -930,7 +1088,7 @@ empathy_event_manager_get_top_event (EmpathyEventManager *manager)
 void
 empathy_event_activate (EmpathyEvent *event_public)
 {
-  EventPriv *event = (EventPriv*) event_public;
+  EventPriv *event = (EventPriv *) event_public;
 
   g_return_if_fail (event_public != NULL);
 
@@ -949,4 +1107,3 @@ empathy_event_inhibit_updates (EmpathyEvent *event_public)
 
   event->inhibit = TRUE;
 }
-
