@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * Authors: Jonny Lamb <jonny.lamb@collabora.co.uk>
  * */
@@ -25,10 +25,8 @@
 
 #include <glib.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <glib/gi18n.h>
 
-#include <libmissioncontrol/mc-account.h>
 #include <telepathy-glib/util.h>
 
 #include "empathy-import-dialog.h"
@@ -37,6 +35,7 @@
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include <libempathy/empathy-debug.h>
 #include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-account-manager.h>
 
 #include <libempathy-gtk/empathy-ui-utils.h>
 
@@ -92,13 +91,21 @@ empathy_import_account_data_free (EmpathyImportAccountData *data)
 static void
 import_dialog_add_account (EmpathyImportAccountData *data)
 {
-  McAccount *account;
+  EmpathyAccountManager *account_manager;
+  EmpathyAccount *account;
   GHashTableIter iter;
   gpointer key, value;
   gchar *display_name;
   GValue *username;
 
-  account = mc_account_create (data->profile);
+  account_manager = empathy_account_manager_dup_singleton ();
+  account = empathy_account_manager_create (account_manager, data->profile);
+  g_object_unref (account_manager);
+  if (account == NULL)
+    {
+      DEBUG ("Failed to create account");
+      return;
+    }
 
   g_hash_table_iter_init (&iter, data->settings);
   while (g_hash_table_iter_next (&iter, &key, &value))
@@ -111,21 +118,21 @@ import_dialog_add_account (EmpathyImportAccountData *data)
           case G_TYPE_STRING:
             DEBUG ("Set param '%s' to '%s' (string)",
                 param, g_value_get_string (gvalue));
-            mc_account_set_param_string (account,
+            empathy_account_set_param_string (account,
                 param, g_value_get_string (gvalue));
             break;
 
           case G_TYPE_BOOLEAN:
             DEBUG ("Set param '%s' to %s (boolean)",
                 param, g_value_get_boolean (gvalue) ? "TRUE" : "FALSE");
-            mc_account_set_param_boolean (account,
+            empathy_account_set_param_boolean (account,
                 param, g_value_get_boolean (gvalue));
             break;
 
           case G_TYPE_INT:
             DEBUG ("Set param '%s' to '%i' (integer)",
                 param, g_value_get_int (gvalue));
-            mc_account_set_param_int (account,
+            empathy_account_set_param_int (account,
                 param, g_value_get_int (gvalue));
             break;
         }
@@ -136,7 +143,7 @@ import_dialog_add_account (EmpathyImportAccountData *data)
   display_name = g_strdup_printf ("%s (%s)",
       mc_profile_get_display_name (data->profile),
       g_value_get_string (username));
-  mc_account_set_display_name (account, display_name);
+  empathy_account_set_display_name (account, display_name);
 
   g_free (display_name);
   g_object_unref (account);
@@ -151,11 +158,12 @@ import_dialog_account_id_in_list (GList *accounts,
   for (l = accounts; l; l = l->next)
     {
       McAccount *account = l->data;
-      gchar *value;
+      gchar *value = NULL;
       gboolean result;
 
-      if (mc_account_get_param_string (account, "account", &value)
-          == MC_ACCOUNT_SETTING_ABSENT)
+      mc_account_get_param_string (account, "account", &value);
+
+      if (value == NULL)
         continue;
 
       result = tp_strdiff (value, account_id);
@@ -347,12 +355,18 @@ import_dialog_destroy_cb (GtkWidget *widget,
   g_slice_free (EmpathyImportDialog, dialog);
 }
 
+gboolean
+empathy_import_dialog_accounts_to_import (void)
+{
+  return empathy_import_pidgin_accounts_to_import ();
+}
+
 void
 empathy_import_dialog_show (GtkWindow *parent,
                             gboolean warning)
 {
   static EmpathyImportDialog *dialog = NULL;
-  GladeXML *glade;
+  GtkBuilder *gui;
   gchar *filename;
   GList *accounts = NULL;
 
@@ -386,21 +400,18 @@ empathy_import_dialog_show (GtkWindow *parent,
 
       return;
     }
-  
+
   /* We have accounts, let's display the window with them */
   dialog = g_slice_new0 (EmpathyImportDialog);
-  dialog->accounts = accounts;  
+  dialog->accounts = accounts;
 
-  filename = empathy_file_lookup ("empathy-import-dialog.glade", "src");
-  glade = empathy_glade_get_file (filename,
-      "import_dialog",
-      NULL,
+  filename = empathy_file_lookup ("empathy-import-dialog.ui", "src");
+  gui = empathy_builder_get_file (filename,
       "import_dialog", &dialog->window,
       "treeview", &dialog->treeview,
       NULL);
 
-  empathy_glade_connect (glade,
-      dialog,
+  empathy_builder_connect (gui, dialog,
       "import_dialog", "destroy", import_dialog_destroy_cb,
       "import_dialog", "response", import_dialog_response_cb,
       NULL);
@@ -408,7 +419,7 @@ empathy_import_dialog_show (GtkWindow *parent,
   g_object_add_weak_pointer (G_OBJECT (dialog->window), (gpointer) &dialog);
 
   g_free (filename);
-  g_object_unref (glade);
+  g_object_unref (gui);
 
   if (parent)
     gtk_window_set_transient_for (GTK_WINDOW (dialog->window), parent);
