@@ -38,8 +38,8 @@
 #include <libebook/e-book.h>
 #include <libnotify/notify.h>
 
+#include <telepathy-glib/dbus.h>
 #include <telepathy-glib/util.h>
-#include <libmissioncontrol/mc-account.h>
 #include <libmissioncontrol/mission-control.h>
 
 #include <libempathy/empathy-idle.h>
@@ -47,6 +47,7 @@
 #include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-account-manager.h>
+#include <libempathy/empathy-debugger.h>
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-dispatch-operation.h>
 #include <libempathy/empathy-log-manager.h>
@@ -105,7 +106,7 @@ dispatch_cb (EmpathyDispatcher *dispatcher,
 		if (id) {
 			EmpathyAccountManager *manager;
 			TpConnection *connection;
-			McAccount *account;
+			EmpathyAccount *account;
 
 			manager = empathy_account_manager_dup_singleton ();
 			connection = empathy_tp_chat_get_connection (tp_chat);
@@ -225,7 +226,8 @@ create_salut_account (void)
 	McProfile  *profile;
 	McProtocol *protocol;
 	gboolean    salut_created = FALSE;
-	McAccount  *account;
+	EmpathyAccount  *account;
+	EmpathyAccountManager *account_manager;
 	GList      *accounts;
 	EBook      *book;
 	EContact   *contact;
@@ -282,15 +284,17 @@ create_salut_account (void)
 		return;
 	}
 
-	account = mc_account_create (profile);
-	mc_account_set_display_name (account, _("People nearby"));
-	
+	account_manager = empathy_account_manager_dup_singleton ();
+	account = empathy_account_manager_create (account_manager, profile);
+	empathy_account_set_display_name (account, _("People nearby"));
+	g_object_unref (account_manager);
+
 	nickname = e_contact_get (contact, E_CONTACT_NICKNAME);
 	first_name = e_contact_get (contact, E_CONTACT_GIVEN_NAME);
 	last_name = e_contact_get (contact, E_CONTACT_FAMILY_NAME);
 	email = e_contact_get (contact, E_CONTACT_EMAIL_1);
 	jid = e_contact_get (contact, E_CONTACT_IM_JABBER_HOME_1);
-	
+
 	if (!tp_strdiff (nickname, "nickname")) {
 		g_free (nickname);
 		nickname = NULL;
@@ -300,11 +304,11 @@ create_salut_account (void)
 		"last-name=%s\nemail=%s\njid=%s\n",
 		nickname, first_name, last_name, email, jid);
 
-	mc_account_set_param_string (account, "nickname", nickname ? nickname : "");
-	mc_account_set_param_string (account, "first-name", first_name ? first_name : "");
-	mc_account_set_param_string (account, "last-name", last_name ? last_name : "");
-	mc_account_set_param_string (account, "email", email ? email : "");
-	mc_account_set_param_string (account, "jid", jid ? jid : "");
+	empathy_account_set_param_string (account, "nickname", nickname ? nickname : "");
+	empathy_account_set_param_string (account, "first-name", first_name ? first_name : "");
+	empathy_account_set_param_string (account, "last-name", last_name ? last_name : "");
+	empathy_account_set_param_string (account, "email", email ? email : "");
+	empathy_account_set_param_string (account, "jid", jid ? jid : "");
 
 	g_free (nickname);
 	g_free (first_name);
@@ -353,7 +357,7 @@ on_bacon_message_received (const char *message,
 				gtk_widget_realize (GTK_WIDGET (window));
 			}
 
-			startup_timestamp = gdk_x11_get_server_time (window->window);
+			startup_timestamp = gdk_x11_get_server_time (gtk_widget_get_window (window));
 		}
 
 		gtk_window_present_with_time (GTK_WINDOW (window), startup_timestamp);
@@ -449,6 +453,31 @@ new_call_handler_cb (EmpathyCallFactory *factory, EmpathyCallHandler *handler,
 	gtk_widget_show (GTK_WIDGET (window));
 }
 
+#ifdef ENABLE_DEBUG
+static void
+default_log_handler (const gchar *log_domain,
+    GLogLevelFlags log_level,
+    const gchar *message,
+    gpointer user_data)
+{
+	g_log_default_handler (log_domain, log_level, message, NULL);
+
+	/* G_LOG_DOMAIN = "empathy". No need to send empathy messages to the
+	 * debugger as they already have in empathy_debug. */
+	if (log_level != G_LOG_LEVEL_DEBUG
+	    || tp_strdiff (log_domain, G_LOG_DOMAIN)) {
+		EmpathyDebugger *dbg;
+		GTimeVal now;
+
+		dbg = empathy_debugger_get_singleton ();
+		g_get_current_time (&now);
+
+		empathy_debugger_add_message (dbg, &now, log_domain,
+					      log_level, message);
+	}
+}
+#endif /* ENABLE_DEBUG */
+
 int
 main (int argc, char *argv[])
 {
@@ -470,6 +499,7 @@ main (int argc, char *argv[])
 	gboolean           hide_contact_list = FALSE;
 	gboolean           accounts_dialog = FALSE;
 	GError            *error = NULL;
+	TpDBusDaemon      *dbus_daemon;
 	GOptionEntry       options[] = {
 		{ "no-connect", 'n',
 		  0, G_OPTION_ARG_NONE, &no_connect,
@@ -512,6 +542,11 @@ main (int argc, char *argv[])
 	gtk_window_set_default_icon_name ("empathy");
 	textdomain (GETTEXT_PACKAGE);
 
+#ifdef ENABLE_DEBUG
+	/* Set up debugger */
+	g_log_set_default_handler (default_log_handler, NULL);
+#endif
+
         /* Setting up the bacon connection */
 	startup_timestamp = get_startup_timestamp ();
 	connection = bacon_message_connection_new ("empathy");
@@ -547,6 +582,23 @@ main (int argc, char *argv[])
 		g_warning ("Cannot create the 'empathy' bacon connection.");
 	}
 
+	/* Take well-known name */
+	dbus_daemon = tp_dbus_daemon_dup (&error);
+	if (error == NULL) {
+		if (!tp_dbus_daemon_request_name (dbus_daemon,
+						  "org.gnome.Empathy",
+						  TRUE, &error)) {
+			DEBUG ("Failed to request well-known name: %s",
+			       error ? error->message : "no message");
+			g_clear_error (&error);
+		}
+		g_object_unref (dbus_daemon);
+	} else {
+		DEBUG ("Failed to dup dbus daemon: %s",
+		       error ? error->message : "no message");
+		g_clear_error (&error);
+	}
+
 	/* Setting up MC */
 	mc = empathy_mission_control_dup_singleton ();
 	g_signal_connect (mc, "ServiceEnded",
@@ -584,7 +636,7 @@ main (int argc, char *argv[])
 			(idle), TP_CONNECTION_PRESENCE_TYPE_OFFLINE) <= 0) {
 		empathy_idle_set_state (idle, MC_PRESENCE_AVAILABLE);
 	}
-	
+
 	create_salut_account ();
 
 	/* Setting up UI */
